@@ -31,20 +31,18 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: irc.c,v 1.126 2006/05/03 16:49:47 f Exp $
+ * $Id: irc.c,v 1.138 2009-12-21 15:08:26 f Exp $
  */
 
-#define IRCII_VERSION	"20060504"	/* YYYYMMDD */
+#define IRCII_VERSION	"20111230"	/* YYYYMMDD */
 /**************************** PATCHED by Flier ******************************/
-#define SCROLLZ_VERSION "1.9.98.1"
+#define SCROLLZ_VERSION "2.2.1"
 /****************************************************************************/
 
 #include "irc.h"
 
 #include <sys/stat.h>
-#ifndef __MSDOS__
 #include <pwd.h>
-#endif /* __MSDOS__ */
 
 #ifdef ISC22
 # include <sys/bsdtypes.h>
@@ -120,7 +118,7 @@ char	oper_command = 0;	/* true just after an oper() command is
 				 * an oper() command and one generated when
 				 * connecting to a new server */
 
-char	FAR MyHostName[80];	       	/* The local machine name. Used by
+char	MyHostName[80];	       		/* The local machine name. Used by
 					 * DCC TALK */
 	struct	in_addr	MyHostAddr;	/* The local machine address */
 	struct	in_addr forced_ip_addr;	/* The local machine address */
@@ -133,9 +131,9 @@ char	*invite_channel = (char *) 0,	/* last channel of an INVITE */
 	*irc_path = (char *) 0,		/* paths used by /load */
 	*irc_lib = (char *) 0,		/* path to the ircII library */
  	*nickname = (char *) 0,		/* users nickname */
-	FAR hostname[NAME_LEN + 1],    	/* name of current host */
-	FAR realname[REALNAME_LEN + 1],	/* real name of user */
-	FAR username[NAME_LEN + 1],    	/* usernameof user */
+	hostname[NAME_LEN + 1],    	/* name of current host */
+	realname[REALNAME_LEN + 1],	/* real name of user */
+	username[NAME_LEN + 1],    	/* usernameof user */
 	*source_host = NULL,		/* specify a specific source host
 					 * for multi-homed machines */
 	*send_umode = NULL,		/* sent umode */
@@ -186,6 +184,7 @@ static	char	*get_arg _((char *, char *, int *));
 static	char	*parse_args _((char **, int));
 
 static	int	cntl_c_hit = 0;
+static	int	connect_at_startup = 1;
 
 /**************************** PATCHED by Flier ******************************
 #ifdef DO_USER2
@@ -195,7 +194,7 @@ static	int	cntl_c_hit = 0;
 
 	char	irc_version[] = IRCII_VERSION;
 
-static	char	FAR switch_help[] =
+static	char	switch_help[] =
 /**************************** Patched by Flier ******************************/
 /*"Usage: irc [switches] [nickname] [server list] \n\*/
 /****************************************************************************/
@@ -216,6 +215,7 @@ static	char	FAR switch_help[] =
    -q\t\tdoes not load .scrollzrc nor .scrollzquick\n\
    -a\t\tadds default servers and command line servers to server list\n\
    -b\t\tload .scrollzrc before connecting to a server\n\
+   -n\t\tdo not connect to a server on startup\n\
    -t\t\tdo not use termcap ti and te sequences at startup\n\
    -T\t\tuse termcap ti and te sequences at startup (default)\n\
    -l <file>\tloads <file> in place of your .scrollzrc\n\
@@ -265,7 +265,7 @@ char  defban;
 char  bold=2;
 char  *DefaultServer=(char *) 0;
 char  *ScrollZstr=(char *) 0;
-char  *ScrollZver="ircII " IRCII_VERSION "+ScrollZ " SCROLLZ_VERSION " (04.05.2006)+Cdcc v" CDCC_VERSION;
+char  *ScrollZver="ircII " IRCII_VERSION "+ScrollZ " SCROLLZ_VERSION " (30.12.2011)+Cdcc v" CDCC_VERSION;
 char  *ScrollZver1=(char *) 0;
 #ifdef EXTRA_STUFF
 char  *EString=(char *) 0;
@@ -355,10 +355,15 @@ char  *ChanLogDir=(char *) 0;
 char  *ChanLogPrefix=(char *) 0;
 char  *ChanLogPostfix=(char *) 0;
 char  *ExtTopicDelimiter=(char *) 0;
+char  *ShowChanChannels=(char *) 0;
 #ifdef BLAXTHOS
 char  *OperNick=(char *) 0;
 char  *OperPassword=(char *) 0;
 #endif
+#ifdef HAVE_GMP
+char  MyPrivKey[mybufsize / 2];
+char  MyPubKey[mybufsize / 2];
+#endif
 int   DeopPrinted;
 int   KickPrinted;
 int   NickPrinted;
@@ -464,6 +469,9 @@ int   ShowNickAllChan;
 int   ExtPub;
 int   ChanLog;
 int   AwayEncrypt;
+int   BanTime;
+int   StatusLines = DEFAULT_STATUS_LINES - 1;
+int   ShowChan;
 #ifdef ACID
 int   ForceJoin;
 #endif
@@ -584,7 +592,7 @@ char VersionInfo[] = {
     'E',
     'R',
 #endif
-#ifdef HAVE_SSL
+#if defined(HAVE_SSL) || defined(HAVE_OPENSSL)
     ' ',
     'S',
     'S',
@@ -597,6 +605,13 @@ char VersionInfo[] = {
     'F',
     '-',
     '8',
+#endif
+#ifdef HAVE_GMP
+    ' ',
+    'F',
+    'i',
+    'S',
+    'H',
 #endif
     '\0'};
 /*****************************/
@@ -616,9 +631,7 @@ irc_exit(quit)
 	close_server(-1, empty_string);
 	logger(0);
 	set_history_file((char *) 0);
-#ifndef _Windows
 	clean_up_processes();
-#endif /* _Windows */
 	if (!dumb)
 	{
 		cursor_to_input();	/* Needed so that ircII doesn't gobble
@@ -641,11 +654,7 @@ irc_exit(quit)
 /**************************** PATCHED by Flier ******************************/
         /*exit(0);*/
         CleanUp();
-#ifdef _Windows
-   DestroyWindow(hwndMain);
-#else
         exit(quit);
-#endif /* _Windows */
 /****************************************************************************/
 }
 
@@ -744,9 +753,7 @@ real_sig_user1()
 {
 	say("Got SIGUSR1, closing DCC connections and EXECed processes");
 	close_all_dcc();
-#ifndef _Windows
 	close_all_exec();
-#endif /* _Windows */
 }
 
 /**************************** PATCHED by Flier ******************************
@@ -869,9 +876,6 @@ parse_args(argv, argc)
 	int	add_servers = 0;
 	char	*channel = (char *) NULL;
 	struct	passwd	*entry;
-#ifdef _Windows
-	u_char	buffer[BIG_BUFFER_SIZE];
-#endif
 /**************************** PATCHED by Flier ******************************/
 	/*int	minus_minus = 0;*/
         char    *CloakCommand = NULL;
@@ -1003,6 +1007,9 @@ parse_args(argv, argc)
 					}
 					qflag = 1;
 					break;
+				case 'n':
+					connect_at_startup = 0;
+					break;
 /**************************** PATCHED by Flier ******************************/
                                 /* what the hell is this */
 				/*case '-':
@@ -1074,12 +1081,8 @@ parse_args(argv, argc)
 	{
 #ifdef IRCPATH
 		malloc_strcpy(&irc_path, IRCPATH);
-#else
-#ifdef __MSDOS__
-		malloc_strcpy(&irc_path, ".:~/irc:");
 #else
 		malloc_strcpy(&irc_path, ".:~/.irc:");
-#endif /* __MSDOS__ */
 		malloc_strcat(&irc_path, irc_lib);
 		malloc_strcat(&irc_path, "script");
 #endif
@@ -1097,29 +1100,11 @@ parse_args(argv, argc)
 		{
  			char *s = (char *) 0;
 
-#ifdef _Windows
-			GetProfileString("IRC", "Server",
-						"Choose.File->Setup.From.Menu",
-						buffer, sizeof(buffer));
- 			malloc_strcpy(&s, buffer);
-#else
  			malloc_strcpy(&s, DEFAULT_SERVER);
-#endif /* _Windows */
  			build_server_list(s);
  			new_free(&s);
 		}
 	}
-#ifdef _Windows
- 	if (nickname == 0 || !*nickname)
- 	{
- 		GetProfileString("IRC", "Nick", "ircuser", buffer, BIG_BUFFER_SIZE);
- 		malloc_strcpy(&nickname, buffer);
- 	}
-	GetProfileString("IRC", "UserName", "ircuser", username, NAME_LEN + 1);
-	GetProfileString("IRC", "RealName", "ircuser", realname, REALNAME_LEN + 1);
-	GetProfileString("IRC", "StartDir", get_path(4), buffer, BIG_BUFFER_SIZE);
-	malloc_strcpy(&my_path, buffer);
-#else /* _Windows */
 	if ((struct passwd *) 0 != (entry = getpwuid(getuid())))
 	{
 		if ((*realname == '\0') && entry->pw_gecos && *(entry->pw_gecos))
@@ -1152,7 +1137,6 @@ parse_args(argv, argc)
 		if (entry->pw_dir && *(entry->pw_dir))
 			malloc_strcpy(&my_path, entry->pw_dir);
 	}
-#endif /* _Windows */
 	if ((char *) 0 != (ptr = getenv("HOME")))
 		malloc_strcpy(&my_path, ptr);
 	else if (*my_path == '\0')
@@ -1232,6 +1216,7 @@ TimerTimeout(struct timeval *tv)
 {
 	struct timeval current;
 /**************************** PATCHED by Flier ******************************/
+        int do_clock, sec_to_minute;
         time_t nickt = LastNick + OrigNickDelay;
 	struct timeval largest;
 /****************************************************************************/
@@ -1241,9 +1226,11 @@ TimerTimeout(struct timeval *tv)
 
 /**************************** PATCHED by Flier ******************************/
         DoOrigNick = 0;
+        do_clock = get_int_var(CLOCK_VAR);
 	/* If ORIGNICK is off set nickt to current + 75 to prevent excessive
 	   CPU usage (meaning we actually ignore this event, see below) */
 	gettimeofday(&current, NULL);
+        if (do_clock) sec_to_minute = (current.tv_sec / 60 + 1) * 60 - current.tv_sec;
         if (!OrigNickChange) nickt = current.tv_sec + 75;
 /****************************************************************************/
 
@@ -1254,6 +1241,7 @@ TimerTimeout(struct timeval *tv)
                 if (nickt - current.tv_sec < 70) DoOrigNick = 1;
                 tv->tv_sec = nickt - current.tv_sec > 70 ? 70 : nickt - current.tv_sec;
                 if (DoOrigNick && tv->tv_sec <= 0) tv->tv_sec = 70;
+                if (do_clock && sec_to_minute < tv->tv_sec) tv->tv_sec = sec_to_minute;
 /****************************************************************************/
 		return;
 	}
@@ -1268,6 +1256,10 @@ TimerTimeout(struct timeval *tv)
             largest.tv_sec = PendingTimers->time;
             largest.tv_usec = PendingTimers->microseconds;
         }
+        if (do_clock && (sec_to_minute < current.tv_sec + largest.tv_sec)) {
+            largest.tv_sec = current.tv_sec + sec_to_minute;
+            largest.tv_usec = 0;
+        }
 /****************************************************************************/
 
 /**************************** PATCHED by Flier ******************************/
@@ -1421,7 +1413,6 @@ irc_io(prompt, func, my_use_input, loop)
 		FD_ZERO(&wd);
 		set_process_bits(&rd);
 		set_server_bits(&rd, &wd);
-#ifndef _Windows
 		if (my_use_input)
 			for (screen = screen_list;screen; screen = screen->next)
 				if (screen->alive)
@@ -1436,7 +1427,6 @@ irc_io(prompt, func, my_use_input, loop)
 			refresh_screen(0, (char *) 0);
 			term_reset_flag = 0;
 		}
-#endif /* _Windows */
 		TimerTimeout(&timer);
 		if (timer.tv_sec <= timeptr->tv_sec)
 			timeptr = &timer;
@@ -1492,13 +1482,11 @@ irc_io(prompt, func, my_use_input, loop)
 /****************************************************************************/
 			break;
 		default:
-#ifndef _Windows
 			if (term_reset_flag)
 			{
 				refresh_screen(0, (char *) 0);
 				term_reset_flag = 0;
 			}
-#endif /* _Windows */
 			old_current_screen = current_screen;
 			set_current_screen(last_input_screen);
 			if (!break_io_processing)
@@ -1591,11 +1579,9 @@ irc_io(prompt, func, my_use_input, loop)
 #ifdef WINDOW_CREATE
 					else
 					{
-#ifndef _Windows
 						if (!is_main_screen(screen))
 							kill_screen(screen);
 						else
-#endif /* _Windows */
 /**************************** PATCHED by Flier ******************************/
 							/*irc_exit();*/
 							irc_exit(0);
@@ -1645,18 +1631,14 @@ irc_io(prompt, func, my_use_input, loop)
 				}
 			}
 			set_current_screen(old_current_screen);
-#ifndef _Windows
 			if (!break_io_processing)
 				do_processes(&rd);
-#endif /* _Windows */
 			break;
 		}
 		execute_timer();
-#ifndef _Windows
 		check_process_limits();
 		while (check_wait_status(-1) >= 0)
 			;
-#endif /* _Windows */
 		if ((primary_server == -1) && !never_connected)
 			do_hook(DISCONNECT_LIST, "%s", nickname);
 		timeptr = &clock_timeout;
@@ -1733,9 +1715,6 @@ irc_io(prompt, func, my_use_input, loop)
 }
 
 int
-#ifdef _Windows
-old_main(int argc, char **argv)
-#else
 /*ARGSUSED*/
 main _((int, char *[], char *[]));
 
@@ -1744,7 +1723,6 @@ main(argc, argv, envp)
 	int	argc;
 	char	*argv[];
 	char	*envp[];
-#endif /* _Windows */
 {
 	char	*channel;
 
@@ -1757,9 +1735,6 @@ main(argc, argv, envp)
         gnutls_global_init();
 #endif
 /****************************************************************************/
-#ifdef _Windows
-	reset_pointers();
-#endif /* _Windows */
 	start_time = time((time_t *)0);
 #ifdef	SOCKS
 	SOCKSinit(argv[0]);
@@ -1774,42 +1749,19 @@ main(argc, argv, envp)
 		cbreak();
 	}
 #endif /* ESIX */
-#ifndef _Windows
 	if ((use_input == 0) && !no_fork)
 	{
 		if (fork())
 			_exit(0);
 	}
-#endif /* _Windows */
-#if defined(ESIX) || defined(_Windows)
+#if defined(ESIX)
 	if (gethostname(hostname, NAME_LEN) == NULL)
 #else
 	if (gethostname(hostname, NAME_LEN))
 #endif /* ESIX */
 	{
-#ifdef _Windows
-		switch(WSAGetLastError())
-	{
-	case WSAEFAULT:
-      		MessageBox(0, "Couldn't get host name", 0, MB_OK);
-		break;
-	case WSANOTINITIALISED:
-		MessageBox(0, "Couldn't get host name", 0, MB_OK);
-		break;
-	case WSAENETDOWN:
-		MessageBox(0, "Couldn't get host name", 0, MB_OK);
-		break;
-	case WSAEINPROGRESS:
-		MessageBox(0, "Couldn't get host name", 0, MB_OK);
-		break;
-	default:
-		break;
-	}
-
-#else
 		fprintf(stderr, "irc: couldn't figure out the name of your machine!\n");
 		exit(1);
-#endif /* _Windows */
 	}
 /**************************** PATCHED by Flier ******************************/
         printf("Process [%d] connected to tty [%s]\n",getpid(),ttyname(0));
@@ -1820,7 +1772,6 @@ main(argc, argv, envp)
 	else
 	{
 		init_screen();
-#ifndef _Windows
 /**************************** PATCHED by Flier ******************************/
 /* Patch for OS/2 EMX */
 /*#if !defined(MUNIX) && !defined(_RT) && !defined(ESIX)*/
@@ -1863,7 +1814,6 @@ main(argc, argv, envp)
 /**************************** PATCHED by Flier ******************************/
 /*#endif*/
 /****************************************************************************/
-#endif /* _Windows */
 	}
 
 /**************************** Patched by Flier ******************************/
@@ -1894,11 +1844,7 @@ main(argc, argv, envp)
 			u_char	*s = (u_char *) 0;
 
 			malloc_strcpy(&s, my_path);
-#ifdef __MSDOS__
-			malloc_strcat(&s, "/ircmotd.red");
-#else
 			malloc_strcat(&s, "/.ircmotd");
-#endif /* __MSDOS__ */
 			if (stat(s, &my_stat))
 			{
 				my_stat.st_atime = 0L;
@@ -1938,7 +1884,8 @@ main(argc, argv, envp)
 		load_ircrc();
 	}
 
-	get_connected(0, 0);
+	if (connect_at_startup)
+		get_connected(0, 0);
         if (channel)
 	{
 		char    *ptr;
@@ -1959,13 +1906,11 @@ main(argc, argv, envp)
 	}
 	idle_time = time(0);
 	set_input(empty_string);
-#ifndef _Windows
 	irc_io(get_string_var(INPUT_PROMPT_VAR), NULL, use_input, irc_io_loop);
 /**************************** PATCHED by Flier ******************************/
         /*irc_exit();*/
         irc_exit(0);
 /****************************************************************************/
-#endif /* _Windows */
  	return 0;
 }
 
