@@ -32,7 +32,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: names.c,v 1.56 2004/12/19 19:04:17 f Exp $
+ * $Id: names.c,v 1.62 2008-11-10 15:28:48 f Exp $
  */
 
 #include "irc.h"
@@ -310,6 +310,7 @@ add_channel(channel, server, connected, copy, key, nowho)
 /**************************** PATCHED by Flier ******************************/
                 new->status = 0;
                 new->gotbans = 0;
+                new->repeatexceptions = 0;
                 new->gotwho = nowho;
 		new->mode = 0;
 		new->limit = 0;
@@ -492,9 +493,10 @@ ChannelList *add_to_channel(channel, nick, server, oper, halfop, voice, userhost
                         ischop = 1;
                         nick++;
                     }
-                    else if (*tmpnick == '&' || *tmpnick == '~') {
+                    else if (*tmpnick == '&' || *tmpnick == '~' || *tmpnick == '!') {
                         nick++;
                     }
+                    else break;
                     tmpnick++;
                 }
                 if (ishalfop && !my_stricmp(nick, get_server_nickname(server)))
@@ -764,6 +766,7 @@ char    *servmodes;
             if (isserver) userhost = NULL;
             *servmodeadd = '\0';
             *tmpbufmode = '\0';
+            *modebuf = '\0';
             *servline = '\0';
             *lastdeop = '\0';
             *lastvoice = '\0';
@@ -811,7 +814,8 @@ char    *servmodes;
                         *mode_string == 'l' || *mode_string == 'm' || *mode_string == 'n' ||
                         *mode_string == 'p' || *mode_string == 'q' || *mode_string == 's' ||
                         *mode_string == 't' || *mode_string == 'c' || *mode_string == 'R' ||
-                        *mode_string == 'I' || *mode_string == 'S') {
+                        *mode_string == 'I' || *mode_string == 'S' || *mode_string == 'g' ||
+                        *mode_string == 'z' || *mode_string == 'Q') {
                         if (compadd != add) {
                             if (compmodelen < compmodemax) {
                                 if (add) *compmodeadd ++= '+';
@@ -846,6 +850,17 @@ char    *servmodes;
 		case 'i':
 			value = MODE_INVITE;
 			break;
+/**************************** PATCHED by Flier ******************************/
+                case 'g':
+                        value = MODE_ALLINVITE;
+                        break;
+                case 'Q':
+                        value = MODE_NOFORWARD;
+                        break;
+                case 'z':
+                        value = MODE_REDUCEMODERATED;
+                        break;
+/****************************************************************************/
 		case 'k':
 			value = MODE_KEY;
 			the_key = next_arg(rest, &rest);
@@ -872,7 +887,7 @@ char    *servmodes;
 				limit_set = 1;
 				if (!(limit = next_arg(rest, &rest)))
 					limit = empty_string;
-				else if (0 == strncmp(limit, zero, 1))
+				else if (0 == strncmp(limit, irczero, 1))
 					limit_reset = 1, limit_set = 0, add = 0;
 			}
 			else
@@ -900,7 +915,7 @@ char    *servmodes;
 				if (add) {
 					/* we can only have one of ohv on a hybrid7 server.
 				   	   +v, +h, -h != +v */
-					if (get_server_version(server) == Server2_11)
+					if (get_server_version(server) == Server2_12)
 						*chop &= ~CHAN_VOICE;
 
 					*chop |= CHAN_HALFOP;
@@ -950,7 +965,7 @@ char    *servmodes;
                         }
 			if (ThisNick) {
 				ThisNick->halfop = add;
-				if (add && get_server_version(server) == Server2_11)
+				if (add && get_server_version(server) == Server2_12)
 					ThisNick->hasvoice = 0;
 			}
 			if (check && tmpjoiner) {
@@ -980,7 +995,7 @@ char    *servmodes;
                                 if (add) {
 					/* we can only have one of ohv on a hybrid7 server.
 					   +h, +o, -o != +h */
-					if (get_server_version(server) == Server2_11)
+					if (get_server_version(server) == Server2_12)
 						*chop &= ~(CHAN_HALFOP | CHAN_VOICE);
 
 					*chop |= CHAN_CHOP;
@@ -1127,7 +1142,7 @@ char    *servmodes;
 			{
 				ThisNick->chanop = add;
 
-				if (add && get_server_version(server) == Server2_11)
+				if (add && get_server_version(server) == Server2_12)
 					ThisNick->halfop = ThisNick->hasvoice = 0;
 			}
 			break;
@@ -1519,7 +1534,7 @@ char    *servmodes;
             if (!chan->gotbans || !chan->gotwho) gotops = 0;
             if (!isitme && minusban && HAS_OPS(*chop) && chan->gotbans && chan->gotwho && !gotops && chan->BKList)
                 CheckPermBans(chan);
-            if (!isitme && gotops && (chan->FriendList || chan->BKList))
+            if (!isitme && gotops)
                 HandleGotOps(mynick, chan);
             if (chan->CompressModes) {
                 *compmodeadd = '\0';
@@ -1667,6 +1682,16 @@ remove_channel(channel, server)
 			tmp = lookup_channel(channel, server, CHAN_UNLINK);
 			if (tmp->window)
 				refnum = tmp->window->refnum;
+/**************************** PATCHED by Flier ******************************/
+                        if (server_list[server].compl_last == tmp->channel ||
+                            server_list[server].compl_channel == tmp ||
+                            server_list[server].compl_next == tmp)
+                        {
+                            server_list[server].compl_last = NULL;
+                            server_list[server].compl_next = NULL;
+                            server_list[server].compl_channel = NULL;
+                        }
+/****************************************************************************/
 			free_channel(&tmp);
 		}
 
@@ -1682,6 +1707,11 @@ remove_channel(channel, server)
 			free_channel(&tmp);
 		}
 		server_list[server].chan_list = (ChannelList *) 0;
+/**************************** PATCHED by Flier ******************************/
+                server_list[server].compl_last = NULL;
+                server_list[server].compl_next = NULL;
+                server_list[server].compl_channel = NULL;
+/****************************************************************************/
 	}
 	update_all_windows();
 }
@@ -1755,6 +1785,13 @@ remove_from_channel(channel, nick, server)
 				new_free(&tmp);*/
                                 remove_nick_from_hash(chan,tmp);
                                 add_to_whowas_buffer(tmp,channel);
+                                if (server_list[curr_scr_win->server].compl_last == tmp->nick ||
+                                    server_list[curr_scr_win->server].compl_next == tmp)
+                                {
+                                    server_list[server].compl_last = NULL;
+                                    server_list[server].compl_next = NULL;
+                                }
+
 /****************************************************************************/
 			}
 		}
@@ -1770,6 +1807,12 @@ remove_from_channel(channel, nick, server)
 				new_free(&tmp);*/
                                 remove_nick_from_hash(chan,tmp);
                                 add_to_whowas_buffer(tmp,chan->channel);
+                                if (server_list[curr_scr_win->server].compl_last == tmp->nick ||
+                                    server_list[curr_scr_win->server].compl_next == tmp)
+                                {
+                                    server_list[server].compl_last = NULL;
+                                    server_list[server].compl_next = NULL;
+                                }
 /****************************************************************************/
 			}
 		}
