@@ -32,7 +32,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: parse.c,v 1.76 2005/08/08 18:50:47 f Exp $
+ * $Id: parse.c,v 1.83 2009-06-15 15:02:26 f Exp $
  */
 
 #include "irc.h"
@@ -102,6 +102,7 @@ extern char *TimeStamp _((int));
 extern void ChannelLogSave _((char *, ChannelList *));
 extern void SendToServer _((void));
 extern int  RateLimitJoin _((int));
+extern char *OpenCreateFile _((char *, int));
 
 extern void e_nick _((char *, char *, char *));
 extern void e_channel _((char *, char *, char *));
@@ -315,6 +316,16 @@ p_topic(from, ArgList)
                 ChannelLogSave(tmpbuf, chan);
             }
         }
+        if (URLCatch) {
+            int numurl;
+            char tmpbuf2[mybufsize];
+            char tmpbuf3[mybufsize / 2];
+            char *filepath = NULL;
+
+            snprintf(tmpbuf3, sizeof(tmpbuf3), "%s topic %s", from, ArgList[0]);
+            filepath = OpenCreateFile("ScrollZ.notepad", 1);
+            numurl = GrabURL(ArgList[1], tmpbuf2, filepath, tmpbuf3);
+        }
         if ((double_ignore(ArgList[0], NULL, IGNORE_CRAP)) == IGNORED) return;
         if ((double_ignore(FromUserHost, ArgList[0], IGNORE_CRAP)) == IGNORED) return;
 /****************************************************************************/				
@@ -339,18 +350,29 @@ p_topic(from, ArgList)
 	}
 	else
 	{
+/**************************** PATCHED by Flier ******************************/
+                int iscrypted;
+                char *cstr = "";
+                char tmpbuf[BIG_BUFFER_SIZE + 1];
+
+                strmcpy(tmpbuf, ArgList[1], sizeof(tmpbuf));
+                iscrypted = DecryptMessage(tmpbuf, ArgList[0]);
+                if (iscrypted == 2) cstr = "[*]";
+                else if (iscrypted) cstr = "[!]";
+/****************************************************************************/
+
 		message_from(ArgList[0], LOG_CRAP);
-		if (do_hook(TOPIC_LIST, "%s %s %s", from, ArgList[0], ArgList[1]))
+		if (do_hook(TOPIC_LIST, "%s %s %s", from, ArgList[0], tmpbuf))
 /**************************** PATCHED by Flier ******************************/
 			/*say("%s has changed the topic on channel %s to %s",
 				from, ArgList[0], ArgList[1]);*/
                 {
                         if (*ArgList[1])
-                            say("%s has set the topic on channel %s to %s",
-                                    from, ArgList[0], ArgList[1]);
+                            say("%s has set the topic on channel %s%s to %s",
+                                    from, cstr, ArgList[0], ArgList[1]);
                         else
-                            say("%s has unset the topic on channel %s",
-                                    from, ArgList[0]);
+                            say("%s has unset the topic on channel %s%s",
+                                    from, cstr, ArgList[0]);
                 }
 /****************************************************************************/
 	}
@@ -398,7 +420,10 @@ p_wall(from, ArgList)
 		else
 		{
 			level = set_lastlog_msg_level(LOG_WALL);
-			if (check_flooding(from, WALL_FLOOD, line) &&
+/**************************** PATCHED by Flier ******************************/
+			/*if (check_flooding(from, WALL_FLOOD, line) &&*/
+			if (check_flooding(from, NULL, WALL_FLOOD, line) &&
+/****************************************************************************/
 					do_hook(WALL_LIST, "%s %s", from, line))
 				put_it("%s#%s#%s %s", high, from, high, line);
 			if (beep_on_level & LOG_WALL)
@@ -455,7 +480,10 @@ p_wallops(from, ArgList)
 	{
 		if (get_int_var(USER_WALLOPS_VAR))
 		{
-			if ((flag != DONT_IGNORE) && (check_flooding(from, WALLOP_FLOOD, line)))
+/**************************** PATCHED by Flier ******************************/
+			/*if ((flag != DONT_IGNORE) && (check_flooding(from, WALLOP_FLOOD, line)))*/
+			if ((flag != DONT_IGNORE) && (check_flooding(from, NULL, WALLOP_FLOOD, line)))
+/****************************************************************************/
 			add_to_whois_queue(from, whois_new_wallops, "%s", line);
 		}
 /**************************** PATCHED by Flier ******************************/
@@ -497,7 +525,7 @@ whoreply(from, ArgList)
         int     show_server = 0;
 /****************************************************************************/
 
-	FILE	*fip;
+	FILE	*fip = NULL;
 	char	buf_data[BUFSIZ];
 
 	if (last_width != get_int_var(CHANNEL_NAME_WIDTH_VAR))
@@ -528,12 +556,14 @@ whoreply(from, ArgList)
  	if (*status == 'S')	/* this only true for the header WHOREPLY */
 	{
 		channel = "Channel";
-		if (((who_mask & WHO_FILE) == 0) || (fopen (who_file, "r")))
+		if (((who_mask & WHO_FILE) == 0) || (fip = fopen (who_file, "r")))
 		{
 			if (do_hook(WHO_LIST, "%s %s %s %s %s %s", channel,
  					nick, status, user, host, ArgList[6]))
  				put_it(format, channel, nick, status, user,
 					host, ArgList[6]);
+			if (fip)
+				fclose(fip);
 			return;
 		}
 	}
@@ -560,6 +590,11 @@ whoreply(from, ArgList)
 		if (who_mask & WHO_CHOPS)
 			ok = ok && ((*(status + 1) == '@') ||
 			(*(status + 2) == '@'));
+/**************************** PATCHED by Flier ******************************/
+                if (who_mask & WHO_NONCHOPS)
+                        ok = ok && ((*(status + 1) != '@') &&
+                        (*(status + 2) != '@'));
+/****************************************************************************/
 		if (who_mask & WHO_HOPS)
 			ok = ok && (*(status + 1) == '%');
 		if (who_mask & WHO_NAME)
@@ -720,11 +755,14 @@ p_privmsg(from, Args)
 		break;
 	}
 /**************************** Patched by Flier ******************************/
-        if (ignore_type == IGNORE_PUBLIC && double_ignore(to, NULL, IGNORE_PUBLIC) == IGNORED)
+        if ((flag != DONT_IGNORE) && (ignore_type == IGNORE_PUBLIC) &&
+            (double_ignore(to, NULL, IGNORE_PUBLIC) == IGNORED))
             goto out;
-        if (ignore_type == IGNORE_PUBLIC && is_channel(to) && double_ignore(from, to, IGNORE_PUBLIC) == IGNORED)
+        if ((flag != DONT_IGNORE) && (ignore_type == IGNORE_PUBLIC) && is_channel(to) &&
+            (double_ignore(from, to, IGNORE_PUBLIC) == IGNORED))
             goto out;
-        if (ignore_type == IGNORE_PUBLIC && is_channel(to) && double_ignore(FromUserHost, to, IGNORE_PUBLIC) == IGNORED)
+        if ((flag != DONT_IGNORE) && (ignore_type == IGNORE_PUBLIC) && is_channel(to) &&
+            (double_ignore(FromUserHost, to, IGNORE_PUBLIC) == IGNORED))
             goto out;
         if (FloodProt > 1 && flood_type == MSG_FLOOD) {
             snprintf(tmpbuf, sizeof(tmpbuf), "%s!%s", from, FromUserHost);
@@ -762,7 +800,10 @@ p_privmsg(from, Args)
 		add_to_whois_queue(from, whois_ignore_msgs, "%s", ptr);
 	else
 	{
-		no_flood = check_flooding(from, flood_type, ptr);
+/**************************** PATCHED by Flier ******************************/
+		/*no_flood = check_flooding(from, flood_type, ptr);*/
+		no_flood = check_flooding(from, to, flood_type, ptr);
+/****************************************************************************/
  		if (sed == 0 || do_hook(ENCRYPTED_PRIVMSG_LIST,"%s %s %s",from, to, ptr))
 		{
 		switch (list_type)
@@ -1231,9 +1272,7 @@ p_channel(from, ArgList)
 			add_channel(channel, parsing_server_index, CHAN_JOINED, NULL, NULL,
                                     chan ? chan->gotwho : 0);
                         chan = lookup_channel(channel, parsing_server_index, 0);
-                        if ((get_server_version(parsing_server_index) == Server2_9 || 
-                             get_server_version(parsing_server_index) == Server2_10 ||
-                             get_server_version(parsing_server_index) == Server2_11) &&
+                        if ((get_server_version(parsing_server_index) == Server2_11) &&
                             IsIrcNetOperChannel(channel)) {
                             snprintf(tmpbuf,sizeof(tmpbuf), "MODE %s", channel);
                         }
@@ -1907,10 +1946,9 @@ irc2_parse_server(line)
 		return;		/* Empty line from server - ByeBye */
 
 	/*
-	 * XXX!!!
-	 * this should fail on '1xxx'!!!
+	 * only allow numbers 1 .. 999.
 	 */
-	if (0 != (numeric = atoi(comm)))
+	if ((numeric = atoi(comm)) > 0 && numeric < 1000)
 		numbered_command(from, numeric, ArgList);
 	else if (strcmp(comm, "NAMREPLY") == 0)
 		funny_namreply(from, ArgList);
