@@ -31,7 +31,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: server.c,v 1.62 2005/09/07 17:55:53 f Exp $
+ * $Id: server.c,v 1.69 2009-12-21 15:24:07 f Exp $
  */
 
 #include "irc.h"
@@ -69,7 +69,7 @@ extern int  CheckServer _((int));
 extern void ChannelLogReportAll _((char *, ChannelList *));
 extern char *OpenCreateFile _((char *, int));
 
-#ifdef HAVE_SSL
+#if defined(HAVE_SSL) || defined(HAVE_OPENSSL)
 int SSLconnect = 0;
 #endif
 /****************************************************************************/
@@ -127,6 +127,9 @@ close_server(server_index, message)
 	int	i,
 		min,
 		max;
+/**************************** Patched by Flier ******************************/
+        int     wasconnected = 0;
+/****************************************************************************/
 
 	if (server_index == -1)
 	{
@@ -154,6 +157,9 @@ close_server(server_index, message)
 		mark_not_connected(i);
 		from_server = old_server;
 
+/**************************** PATCHED by Flier ******************************/
+                wasconnected = server_list[i].connected;
+/****************************************************************************/
 		server_list[i].operator = 0;
 		server_list[i].connected = 0;
 		server_list[i].buffer = (char *) 0;
@@ -180,13 +186,17 @@ close_server(server_index, message)
 			{
 				snprintf(buffer, sizeof buffer, "QUIT :%s\n", message);
 /**************************** Patched by Flier ******************************/
-#ifdef HAVE_SSL
-                                if (server_list[i].connected &&
-                                    server_list[i].enable_ssl &&
-                                    server_list[i].session) {
-                                    if (gnutls_transport_get_ptr(server_list[i].session))
+#if defined(HAVE_SSL) || defined(HAVE_OPENSSL)
+                                if (wasconnected && server_list[i].enable_ssl) {
+#if defined(HAVE_SSL)
+                                    if (server_list[i].session && gnutls_transport_get_ptr(server_list[i].session))
                                         gnutls_record_send(server_list[i].session,
                                                            buffer, strlen(buffer));
+#elif defined(HAVE_OPENSSL)
+                                    if (server_list[i].ssl_fd)
+                                        SSL_write(server_list[i].ssl_fd, buffer, strlen(buffer));
+
+#endif
                                 }
                                 else
 #endif
@@ -203,21 +213,30 @@ close_server(server_index, message)
 			new_close(server_list[i].read);
 			server_list[i].read = -1;
 		}
-#ifndef _Windows
 		if (-1 != server_list[i].pid)
 		{
 			kill(server_list[i].pid, SIGKILL);
 			server_list[i].pid = (pid_t) -1;
 		}
-#endif /* _Windows */
 /**************************** Patched by Flier ******************************/
-#ifdef HAVE_SSL
-                if (server_list[i].enable_ssl && server_list[i].session &&
-                    gnutls_transport_get_ptr(server_list[i].session)) {
-                    gnutls_bye(server_list[i].session, GNUTLS_SHUT_RDWR);
-                    gnutls_deinit(server_list[i].session);
-                    gnutls_certificate_free_credentials(server_list[i].xcred);
-                    memset(&server_list[from_server].session, 0, sizeof(gnutls_session));
+#if defined(HAVE_SSL) || defined(HAVE_OPENSSL)
+                if (server_list[i].enable_ssl) {
+#if defined(HAVE_SSL)
+                    if (server_list[i].session && gnutls_transport_get_ptr(server_list[i].session)) {
+                        gnutls_bye(server_list[i].session, GNUTLS_SHUT_RDWR);
+                        gnutls_deinit(server_list[i].session);
+                        gnutls_certificate_free_credentials(server_list[i].xcred);
+                        server_list[i].session = NULL;
+                        server_list[i].xcred = NULL;
+                    }
+#elif defined(HAVE_OPENSSL)
+                    if (server_list[i].ssl_fd) {
+                        SSL_shutdown(server_list[i].ssl_fd);
+                        server_list[i].ssl_fd = NULL;
+                        server_list[i].ctx = NULL;
+                        server_list[i].meth = NULL;
+                    }
+#endif
                 }
 #endif
 /****************************************************************************/
@@ -309,10 +328,14 @@ do_server(rd, wd)
 			if (len >= BIG_BUFFER_SIZE)
 				goto buffer_is_full_hack;	/* XXX? */
 /**************************** PATCHED by Flier ******************************/
-#ifdef HAVE_SSL
+#if defined(HAVE_SSL) || defined(HAVE_OPENSSL)
                         if (server_list[from_server].enable_ssl)
                             junk = SSL_dgets(bufptr, BIG_BUFFER_SIZE, des, (char *) 0,
+#if defined(HAVE_SSL)
                                              &server_list[from_server].session);
+#elif defined(HAVE_OPENSSL)
+                                             server_list[from_server].ssl_fd);
+#endif
                         else
 #endif /* HAVE_SSL */
 /****************************************************************************/
@@ -547,12 +570,16 @@ add_to_server_list(server, port, password, nick, overwrite)
 		server_list[from_server].whois = 0;
 		server_list[from_server].flags = SERVER_2_6_2;
 /**************************** PATCHED by Flier ******************************/
-#ifdef HAVE_SSL
+#if defined(HAVE_SSL) || defined(HAVE_OPENSSL)
                 server_list[from_server].enable_ssl = 0;
-                memset(&server_list[from_server].session, 0,
-                       sizeof(gnutls_session));
-                memset(&server_list[from_server].xcred, 0,
-                       sizeof(gnutls_certificate_credentials));
+#if defined(HAVE_SSL)
+                server_list[from_server].session = NULL;
+                server_list[from_server].xcred = NULL;
+#elif defined(HAVE_OPENSSL)
+                server_list[from_server].ssl_fd = NULL;
+                server_list[from_server].ctx = NULL;
+                server_list[from_server].meth = NULL;
+#endif
                 /* this is true only when adding server(s) from a command line */
                 if (*server == '!') {
                     server++;
@@ -573,6 +600,9 @@ add_to_server_list(server, port, password, nick, overwrite)
                 server_list[from_server].nicklist = NULL;
                 server_list[from_server].ConnectTime = 0;
                 server_list[from_server].ChanPendingList = NULL;
+                server_list[from_server].compl_last = NULL;
+                server_list[from_server].compl_next = NULL;
+                server_list[from_server].compl_channel = NULL;
 /****************************************************************************/
 		server_list[from_server].nickname = (char *) 0;
 		server_list[from_server].connected = 0;
@@ -709,6 +739,10 @@ remove_from_server_list(i)
 	if (server_list[i].ctcp_send_size)
 		new_free(&server_list[i].ctcp_send_size);
 /**************************** PATCHED by Flier ******************************/
+#if defined(HAVE_OPENSSL)
+        SSL_CTX_free(server_list[i].ctx);
+        new_free(&server_list[i].meth);
+#endif
         if (server_list[i].LastMessage) new_free(&(server_list[i].LastMessage));
         if (server_list[i].LastNotice) new_free(&(server_list[i].LastNotice));
         if (server_list[i].LastMessageSent) new_free(&(server_list[i].LastMessageSent));
@@ -740,9 +774,7 @@ remove_from_server_list(i)
 	/* update all the structs with server in them */
 	channel_server_delete(i);	/* fix `higher' servers */
 	clear_channel_list(i);
-#ifndef _Windows
 	exec_server_delete(i);
-#endif /* _Windows */
 	if (i < primary_server)
 		--primary_server;
 	if (i < from_server)
@@ -956,7 +988,7 @@ connect_to_server_direct(server_name, port, nick)
 /**************************** PATCHED by Flier ******************************/
 		/*new_des = connect_by_number(port, server_name, 1);*/
         {
-#ifdef HAVE_SSL
+#if defined(HAVE_SSL) || defined(HAVE_OPENSSL)
                 if (*server_name == '!') server_name++;
 #endif
 		new_des = connect_by_number(port, server_name, 1, 0);
@@ -1045,9 +1077,6 @@ connect_to_server_process(server_name, port, nick)
 	int	port;
 	char	*nick;
 {
-#ifdef _Windows
-	return -1;
-#else
 	int	write_des[2],
 		read_des[2],
 		pid,
@@ -1142,7 +1171,6 @@ connect_to_server_process(server_name, port, nick)
 	server_list[from_server].pid = pid;
 	server_list[from_server].operator = 0;
 	return (0);
-#endif /* _Windows */
 }
 
 /*
@@ -1270,7 +1298,7 @@ connect_to_server(server_name, port, nick, c_server)
 			attempting_to_connect = 0;
  			restore_message_from();
 /**************************** Patched by Flier ******************************/
-#ifdef HAVE_SSL
+#if defined(HAVE_SSL) || defined(HAVE_OPENSSL)
                         SSLconnect = 0;
                         if (*server_name == '!') server_list[server_index].enable_ssl = 1;
 #endif
@@ -1302,7 +1330,7 @@ connect_to_server(server_name, port, nick, c_server)
 			server_list[from_server].close_serv = -1;
 		}
 /**************************** Patched by Flier ******************************/
-#ifdef HAVE_SSL
+#if defined(HAVE_SSL) || defined(HAVE_OPENSSL)
                 if (SSLconnect || *server_name == '!' || server_list[from_server].enable_ssl) {
                     server_list[from_server].enable_ssl = 1;
                     server_list[from_server].flags |= SSL_CONNECT;
@@ -1368,6 +1396,9 @@ login_to_server(server)
 	server_list[server].SZWho = 0;
 	server_list[server].SZUnban = 0;
 	server_list[server].ConnectTime = time(NULL);
+	server_list[server].compl_last = NULL;
+	server_list[server].compl_next = NULL;
+	server_list[server].compl_channel = NULL;
 /****************************************************************************/
 	server_list[server].flags |= LOGGED_IN;
 #ifdef NON_BLOCKING_CONNECTS
@@ -1397,27 +1428,44 @@ login_to_server(server)
 #endif
 #endif /* NON_BLOCKING_CONNECTS */
 /**************************** Patched by Flier ******************************/
-#ifdef HAVE_SSL
+#if defined(HAVE_SSL) || defined(HAVE_OPENSSL)
         if (server_list[server].enable_ssl && (server_list[server].flags & SSL_CONNECT)) {
             int err;
-            int cert_type_priority[3] = { GNUTLS_CRT_X509,
-                                          GNUTLS_CRT_OPENPGP, 0 };
+#if defined(HAVE_SSL)
             char *filepath;
+            const char *errpos;
 
             filepath = OpenCreateFile("ca.pem", 1);
+#endif
             say("SSL connect in progress ...");
+#if defined(HAVE_SSL)
             gnutls_certificate_allocate_credentials(&server_list[server].xcred);
             gnutls_certificate_set_x509_trust_file(server_list[server].xcred,
                                                    filepath, GNUTLS_X509_FMT_PEM);
             gnutls_init(&server_list[server].session, GNUTLS_CLIENT);
             gnutls_set_default_priority(server_list[server].session);
-            gnutls_certificate_type_set_priority(server_list[server].session,
-                                                 cert_type_priority);
+            err = gnutls_priority_set_direct(server_list[server].session,
+                                             "NORMAL:COMPAT", &errpos);
+            if (err != GNUTLS_E_SUCCESS) {
+                say("Error setting priorities: %s", errpos ? errpos : "<null>");
+            }
             gnutls_credentials_set(server_list[server].session, GNUTLS_CRD_CERTIFICATE,
                                    server_list[server].xcred);
             gnutls_transport_set_ptr(server_list[server].session,
-                                     (gnutls_transport_ptr) server_list[server].read);
+                                     (gnutls_transport_ptr_t) server_list[server].read);
             err = gnutls_handshake(server_list[server].session);
+#elif defined(HAVE_OPENSSL)
+            SSLeay_add_ssl_algorithms();
+            server_list[server].meth = SSLv3_client_method();
+            SSL_load_error_strings();
+            server_list[server].ctx = SSL_CTX_new(server_list[server].meth);
+            CHK_NULL(server_list[server].ctx);
+            server_list[server].ssl_fd = SSL_new(server_list[server].ctx);
+            CHK_NULL(server_list[server].ssl_fd);
+            SSL_set_fd(server_list[server].ssl_fd, server_list[server].read);
+            err = SSL_connect(server_list[server].ssl_fd);
+            CHK_SSL(err);
+#endif
 
             server_list[server].flags &= ~SSL_CONNECT;
         }
@@ -1583,7 +1631,7 @@ display_server_list()
 /**************************** Patched by Flier ******************************/
 				/*say("\t%d) %s %d%s", i,*/
 				say("\t%c%d) %s %d%s%s",
-#ifdef HAVE_SSL
+#if defined(HAVE_SSL) || defined(HAVE_OPENSSL)
                                         server_list[i].enable_ssl ? '!' : ' ',
 #else
                                         ' ',
@@ -1604,7 +1652,7 @@ display_server_list()
 /**************************** Patched by Flier ******************************/
 					/*say("\t%d) %s %d (was %s)", i,*/
 					say("\t%c%d) %s %d (was %s)",
-#ifdef HAVE_SSL
+#if defined(HAVE_SSL) || defined(HAVE_OPENSSL)
                                                 server_list[i].enable_ssl ? '!' : ' ',
 #else
                                                 ' ',
@@ -1618,7 +1666,7 @@ display_server_list()
 /**************************** Patched by Flier ******************************/
 					/*say("\t%d) %s %d (%s)", i,*/
 					say("\t%c%d) %s %d (%s)%s",
-#ifdef HAVE_SSL
+#if defined(HAVE_SSL) || defined(HAVE_OPENSSL)
                                                 server_list[i].enable_ssl ? '!' : ' ',
 #else
                                                 ' ',
@@ -1726,7 +1774,7 @@ servercmd(command, args, subargs)
 		new_server_flags;
 
 /**************************** Patched by Flier ******************************/
-#ifdef HAVE_SSL
+#if defined(HAVE_SSL) || defined(HAVE_OPENSSL)
         SSLconnect = 0;
 #endif
 /****************************************************************************/
@@ -1774,7 +1822,7 @@ servercmd(command, args, subargs)
 				return;
 			}
 /**************************** Patched by Flier ******************************/
-#ifdef HAVE_SSL
+#if defined(HAVE_SSL) || defined(HAVE_OPENSSL)
                         else if (!strncmp(server, "SSL", len)) SSLconnect = 1;
 #endif
 /****************************************************************************/
@@ -2167,6 +2215,14 @@ int server_index;
 }
 /****************************************************************************/
 
+/**************************** PATCHED by Flier ******************************/
+int is_server_ok(int server_index) {
+    if (server_list == NULL) return(0);
+    if (is_server_valid(server_index) == 0) return(0);
+    return(1);
+}
+/****************************************************************************/
+
 /*
  * is_server_connected: returns true if the given server is connected.  This
  * means that both the tcp connection is open and the user is properly
@@ -2179,7 +2235,7 @@ is_server_connected(server_index)
 /**************************** PATCHED by Flier ******************************/
 	/*if (server_index < 0)
 		return (0);*/
-        if (server_index < 0 || server_index >= number_of_servers) return(0);
+        if (!is_server_ok(server_index)) return(0);
 /****************************************************************************/
 	return (server_list[server_index].connected && (server_list[server_index].flags & LOGGED_IN));
 }
@@ -2191,6 +2247,9 @@ get_server_port(server_index)
 {
 	if (server_index == -1)
 		server_index = primary_server;
+/**************************** PATCHED by Flier ******************************/
+        if (!is_server_ok(server_index)) return(0);
+/****************************************************************************/
 	return (server_list[server_index].port);
 }
 
@@ -2203,7 +2262,7 @@ get_server_nickname(server_index)
 	int	server_index;
 {
 /**************************** PATCHED by Flier ******************************/
-        if (server_index >= number_of_servers)
+        if (!is_server_ok(server_index))
             return(nickname);
 /****************************************************************************/
 	if ((server_index != -1) && server_list[server_index].nickname)
@@ -2220,7 +2279,14 @@ get_server_qhead(server_index)
 	int	server_index;
 {
 	if (server_index != -1)
+/**************************** PATCHED by Flier ******************************/
+        {
+                if (!is_server_ok(server_index)) return(0);
+/****************************************************************************/
 		return server_list[server_index].WQ_head;
+/**************************** PATCHED by Flier ******************************/
+        }
+/****************************************************************************/
 	else
 		return WQ_head;
 }
@@ -2241,7 +2307,14 @@ get_server_qtail(server_index)
 	int	server_index;
 {
 	if (server_index !=-1)
+/**************************** PATCHED by Flier ******************************/
+        {
+                if (!is_server_ok(server_index)) return(0);
+/****************************************************************************/
 		return server_list[server_index].WQ_tail;
+/**************************** PATCHED by Flier ******************************/
+        }
+/****************************************************************************/
 	else
 		return WQ_tail;
 }
@@ -2255,7 +2328,14 @@ set_server_qhead(server_index, value)
 	WhoisQueue *value;
 {
 	if (server_index != -1)
+/**************************** PATCHED by Flier ******************************/
+        {
+                if (!is_server_ok(server_index)) return;
+/****************************************************************************/
 		server_list[server_index].WQ_head = value;
+/**************************** PATCHED by Flier ******************************/
+        }
+/****************************************************************************/
 	else
 		WQ_head = value;
 }
@@ -2267,7 +2347,14 @@ set_server_qtail(server_index, value)
 	WhoisQueue *value;
 {
 	if (server_index !=-1)
+/**************************** PATCHED by Flier ******************************/
+        {
+                if (!is_server_ok(server_index)) return;
+/****************************************************************************/
 		server_list[server_index].WQ_tail = value;
+/**************************** PATCHED by Flier ******************************/
+        }
+/****************************************************************************/
 	else
 		WQ_tail = value;
 }
@@ -2282,6 +2369,9 @@ int
 get_server_operator(server_index)
 	int	server_index;
 {
+/**************************** PATCHED by Flier ******************************/
+        if (!is_server_ok(server_index)) return(0);
+/****************************************************************************/
 	return (server_list[server_index].operator);
 }
 
@@ -2294,6 +2384,9 @@ set_server_operator(server_index, flag)
 	int	server_index;
 	int	flag;
 {
+/**************************** PATCHED by Flier ******************************/
+        if (!is_server_ok(server_index)) return;
+/****************************************************************************/
 	server_list[server_index].operator = flag;
 }
 
@@ -2309,9 +2402,15 @@ set_server_nickname(server_index, nick)
 {
 	if (server_index != -1)
 	{
+/**************************** PATCHED by Flier ******************************/
+            if (is_server_ok(server_index)) {
+/****************************************************************************/
 		malloc_strcpy(&(server_list[server_index].nickname), nick);
 		if (server_index == primary_server)
  			malloc_strcpy(&nickname, nick);
+/**************************** PATCHED by Flier ******************************/
+            }
+/****************************************************************************/
 	}
 	update_all_status();
 }
@@ -2321,7 +2420,10 @@ set_server_motd(server_index, flag)
 	int	server_index;
 	int	flag;
 {
-	if (server_index != -1)
+/**************************** PATCHED by Flier ******************************/
+	/*if (server_index != -1)*/
+        if (!is_server_ok(server_index)) return;
+/****************************************************************************/
 		server_list[server_index].motd = flag;
 }
 
@@ -2329,7 +2431,10 @@ int
 get_server_motd(server_index)
 	int	server_index;
 {
-	if (server_index != -1)
+/**************************** PATCHED by Flier ******************************/
+	/*if (server_index != -1)*/
+        if (!is_server_ok(server_index)) return(0);
+/****************************************************************************/
 		return(server_list[server_index].motd);
 	return (0);
 }
@@ -2339,6 +2444,9 @@ server_is_connected(server_index, value)
 	int	server_index,
 		value;
 {
+/**************************** PATCHED by Flier ******************************/
+        if (!is_server_ok(server_index)) return;
+/****************************************************************************/
 	server_list[server_index].connected = value;
 	if (value)
 		server_list[server_index].eof = 0;
@@ -2405,7 +2513,7 @@ send_to_server(format, arg1, arg2, arg3, arg4, arg5,
 		if (do_hook(RAW_SEND_LIST, "%s", lbuf))
 		{
 /**************************** Patched by Flier ******************************/
-#ifdef HAVE_SSL
+#if defined(HAVE_SSL)
                         if (server_list[server].enable_ssl &&
                             server_list[server].session) {
                             int err;
@@ -2416,7 +2524,18 @@ send_to_server(format, arg1, arg2, arg3, arg4, arg5,
                             }
                             err = gnutls_record_send(server_list[server].session,
                                                      lbuf, len);
+                        }
+                        else
+#elif defined(HAVE_OPENSSL)
+                        if (server_list[server].enable_ssl) {
+                            int err;
+
+                            if (!server_list[server].ssl_fd) {
+                                say("SSL write error - ssl socket = 0");
+                                return;
                             }
+                            err = SSL_write(server_list[server].ssl_fd, lbuf, len);
+                        }
                         else
 #endif
 /****************************************************************************/
