@@ -31,7 +31,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: vars.c,v 1.27 2006/04/30 14:15:43 f Exp $
+ * $Id: vars.c,v 1.39 2009-12-21 14:14:17 f Exp $
  */
 
 #include "irc.h"
@@ -59,20 +59,9 @@
 extern char *HelpPathVar;
 extern char *TimeStampString;
 extern time_t LastTS;
+extern struct in_addr forced_ip_addr;
 /****************************************************************************/
 
-/* IrcVariable: structure for each variable in the variable table */
-typedef struct
-{
-	char	*name;			/* what the user types */
-	int	type;			/* variable types, see below */
-	int	integer;		/* int value of variable */
-	char	*string;		/* string value of variable */
- 	void	(*func) ();		/* function to do every time variable is set */
-	char	int_flags;		/* internal flags to the variable */
-	unsigned short	flags;		/* flags for this variable */
-}	IrcVariable;
-
 #define	VF_NODAEMON	0x0001
 #define VF_EXPAND_PATH	0x0002
 
@@ -97,6 +86,7 @@ extern  Screen	*screen_list, *current_screen;
 
 	int	loading_global = 0;
 
+static  void	check_variable_order _((void));
 static	int	find_variable _((char *, int *));
 static	void	exec_warning _((int));
 static	void	input_warning _((int));
@@ -108,10 +98,13 @@ static  void    SetScrollZstr _((char *));
 static  void    SetMaxModes _((int));
 static  void    SetMaxWallopNicks _((int));
 static  void    SetDCCBlockSize _((int));
+static  void    SetDCCHost _((char *));
 static  void    SetDCCPorts _((char *));
 static	void	Cnotifystring _((char *));
 static  void    SetAwayFile _((char *));
 void    SetStampFormat _((char *));
+static  void    SetStatusLines _((int));
+static  void    SetUsername _((char *));
 
 extern  void    RedrawAll _((void));
 extern  void    UpdateFloodUsers _((void));
@@ -121,20 +114,21 @@ extern  int     DCCHighPort;
 extern	char	*CelerityNtfy;
 /****************************************************************************/
 
-#ifdef _Windows
-extern	char *get_ini_value(char *pchEntry);
-#endif /* _Windows */
-
 /*
  * irc_variable: all the irc variables used.  Note that the integer and
  * boolean defaults are set here, which the string default value are set in
- * the init_variables() procedure 
+ * the init_variables() procedur .
+ *
+ * this structure needs to be in alphabetical order, and the
+ * check_variable_order() procedure will assert that it is from
+ * init_variables()
  */
-static	IrcVariable irc_variable[] =
+IrcVariable irc_variable[] =
 {
 	{ "ALWAYS_SPLIT_BIGGEST",	BOOL_TYPE_VAR,	DEFAULT_ALWAYS_SPLIT_BIGGEST, NULL, NULL, 0, 0 },
 /**************************** PATCHED by Flier ******************************/
 	{ "AUTO_RECONNECT",		BOOL_TYPE_VAR,	DEFAULT_AUTO_RECONNECT, NULL, NULL, 0, 0 },
+	{ "AUTO_RECONNECT_CHANNELS",	BOOL_TYPE_VAR,	DEFAULT_AUTO_RECONNECT_CHANNELS, NULL, NULL, 0, 0 },
 /****************************************************************************/
 	{ "AUTO_UNMARK_AWAY",		BOOL_TYPE_VAR,	DEFAULT_AUTO_UNMARK_AWAY, NULL, NULL, 0, 0 },
 	{ "AUTO_WHOWAS",		BOOL_TYPE_VAR,	DEFAULT_AUTO_WHOWAS, NULL, NULL, 0, 0 },
@@ -149,6 +143,9 @@ static	IrcVariable irc_variable[] =
 	{ "BEEP_ON_MSG",		STR_TYPE_VAR,	0, NULL, set_beep_on_msg, 0, 0 },
 	{ "BEEP_WHEN_AWAY",		INT_TYPE_VAR,	DEFAULT_BEEP_WHEN_AWAY, NULL, NULL, 0, 0 },
 	{ "BOLD_VIDEO",			BOOL_TYPE_VAR,	DEFAULT_BOLD_VIDEO, NULL, NULL, 0, 0 },
+/**************************** PATCHED by Flier ******************************/
+	{ "CHANLOG_STRIP_ANSI",		BOOL_TYPE_VAR,	DEFAULT_CHANLOG_STRIP_ANSI, NULL, NULL, 0, 0 },
+/****************************************************************************/
 	{ "CHANNEL_NAME_WIDTH",		INT_TYPE_VAR,	DEFAULT_CHANNEL_NAME_WIDTH, NULL, update_all_status, 0, 0 },
 	{ "CLIENT_INFORMATION",		STR_TYPE_VAR,	0, NULL, NULL, 0, 0 },
 	{ "CLOCK",			BOOL_TYPE_VAR,	DEFAULT_CLOCK, NULL, update_all_status, 0, 0 },
@@ -163,6 +160,7 @@ static	IrcVariable irc_variable[] =
 /**************************** PATCHED by Flier ******************************/
 /*        { "DCC_BLOCK_SIZE",		INT_TYPE_VAR,	DEFAULT_DCC_BLOCK_SIZE, NULL, NULL, 0, 0 },*/
 	{ "DCC_BLOCK_SIZE",		INT_TYPE_VAR,	DEFAULT_DCC_BLOCK_SIZE, NULL, SetDCCBlockSize, 0, 0 },
+	{ "DCC_HOST",		        STR_TYPE_VAR,	0, NULL, SetDCCHost, 0, 0 },
 	{ "DCC_PORTS",		        STR_TYPE_VAR,	0, NULL, SetDCCPorts, 0, 0 },
 /****************************************************************************/
 	{ "DEBUG",			INT_TYPE_VAR,	0, NULL, NULL, 0, 0 },
@@ -173,6 +171,10 @@ static	IrcVariable irc_variable[] =
 /****************************************************************************/
 	{ "DISPLAY_ENCODING",		STR_TYPE_VAR,	0, NULL, set_display_encoding, 0, 0 },
 	{ "EIGHT_BIT_CHARACTERS",	BOOL_TYPE_VAR,	DEFAULT_EIGHT_BIT_CHARACTERS, NULL, eight_bit_characters, 0, 0 },
+/**************************** PATCHED by Flier ******************************/
+	{ "ENCRYPT_PAD_MSGS",		BOOL_TYPE_VAR,	DEFAULT_DISPLAY_ANSI, NULL, 0, 0, 0 },
+	{ "ENCRYPT_PAD_PUBLIC",		BOOL_TYPE_VAR,	DEFAULT_DISPLAY_ANSI, NULL, 0, 0, 0 },
+/****************************************************************************/
 	{ "ENCRYPT_PROGRAM",		STR_TYPE_VAR,	0, NULL, NULL, 0, VF_NODAEMON },
 	{ "EXEC_PROTECTION",		BOOL_TYPE_VAR,	DEFAULT_EXEC_PROTECTION, NULL, exec_warning, 0, VF_NODAEMON },
 	{ "FLOOD_AFTER",		INT_TYPE_VAR,	DEFAULT_FLOOD_AFTER, NULL, NULL, 0, 0 },
@@ -213,6 +215,9 @@ static	IrcVariable irc_variable[] =
 	{ "INVERSE_VIDEO",		BOOL_TYPE_VAR,	DEFAULT_INVERSE_VIDEO, NULL, NULL, 0, 0 },
 	{ "IRCHOST",			STR_TYPE_VAR,	0, NULL, set_irchost, 0, 0 },
 	{ "IRC_ENCODING",		STR_TYPE_VAR,	0, NULL, set_irc_encoding, 0, 0 },
+/**************************** PATCHED by Flier ******************************/
+        { "ISO2022_SUPPORT",		BOOL_TYPE_VAR,	DEFAULT_ISO2022_SUPPORT, NULL, NULL, 0, 0 },
+/****************************************************************************/
 	{ "LASTLOG",			INT_TYPE_VAR,	DEFAULT_LASTLOG, NULL, set_lastlog_size, 0, 0 },
 /**************************** PATCHED by Flier ******************************/
         { "LASTLOG_ANSI",		BOOL_TYPE_VAR,	1, NULL, NULL, 0, 0 },
@@ -237,25 +242,34 @@ static	IrcVariable irc_variable[] =
 #endif
 	{ "MINIMUM_SERVERS",		INT_TYPE_VAR,	DEFAULT_MINIMUM_SERVERS, NULL, NULL, 0, VF_NODAEMON },
 	{ "MINIMUM_USERS",		INT_TYPE_VAR,	DEFAULT_MINIMUM_USERS, NULL, NULL, 0, VF_NODAEMON },
-	{ "NO_ASK_NICKNAME",		BOOL_TYPE_VAR,	DEFAULT_NO_ASK_NICKNAME, NULL, NULL, 0, 0 },
-	{ "NO_CTCP_FLOOD",		BOOL_TYPE_VAR,	DEFAULT_NO_CTCP_FLOOD, NULL, NULL, 0, 0 },
+	{ "NETSPLIT_TIME",		INT_TYPE_VAR,	DEFAULT_NETSPLIT_TIME, NULL, NULL, 0, 0 },
 	{ "NOTIFY_HANDLER",		STR_TYPE_VAR, 	0, 0, set_notify_handler, 0, 0 },
 	{ "NOTIFY_LEVEL",		STR_TYPE_VAR,	0, NULL, set_notify_level, 0, 0 },
 	{ "NOTIFY_ON_TERMINATION",	BOOL_TYPE_VAR,	DEFAULT_NOTIFY_ON_TERMINATION, NULL, NULL, 0, VF_NODAEMON },
 /**************************** PATCHED by Flier ******************************/
+	{ "NOTIFY_SHOW_NAME",		BOOL_TYPE_VAR,	DEFAULT_NOTIFY_SHOW_NAME, NULL, NULL, 0, 0 },
 	{ "NOTIFY_STRING",		STR_TYPE_VAR,	0, NULL, Cnotifystring, 0, 0 },
 /****************************************************************************/
 	{ "NOVICE",			BOOL_TYPE_VAR,	DEFAULT_NOVICE, NULL, NULL, 0, 0 },
+	{ "NO_ASK_NICKNAME",		BOOL_TYPE_VAR,	DEFAULT_NO_ASK_NICKNAME, NULL, NULL, 0, 0 },
+	{ "NO_CTCP_FLOOD",		BOOL_TYPE_VAR,	DEFAULT_NO_CTCP_FLOOD, NULL, NULL, 0, 0 },
 	{ "OLD_ENCRYPT_PROGRAM",	BOOL_TYPE_VAR,	0, NULL, NULL, 0, VF_NODAEMON },
+	{ "RATE_LIMIT_JOIN",		STR_TYPE_VAR,	0, NULL, NULL, 0, 0 },
 	{ "REALNAME",			STR_TYPE_VAR,	0, 0, set_realname, 0, VF_NODAEMON },
 	{ "SAME_WINDOW_ONLY",		BOOL_TYPE_VAR,	DEFAULT_SAME_WINDOW_ONLY, NULL, NULL, 0, 0 },
+/**************************** PATCHED by Flier ******************************/
+        { "SAVE_ENCRYPTION_KEYS",		BOOL_TYPE_VAR,	0, NULL, NULL, 0, 0 },
+/****************************************************************************/
 	{ "SCREEN_OPTIONS", 		STR_TYPE_VAR,	0, NULL, NULL, 0, VF_NODAEMON },
 	{ "SCROLL",			BOOL_TYPE_VAR,	DEFAULT_SCROLL, NULL, set_scroll, 0, 0 },
-	{ "SCROLL_LINES",		INT_TYPE_VAR,	DEFAULT_SCROLL_LINES, NULL, set_scroll_lines, 0, 0 },
 /**************************** PATCHED by Flier ******************************/
         { "SCROLLZ_STRING",             STR_TYPE_VAR,   0, NULL, SetScrollZstr, 0, 0 },
 /****************************************************************************/
+	{ "SCROLL_LINES",		INT_TYPE_VAR,	DEFAULT_SCROLL_LINES, NULL, set_scroll_lines, 0, 0 },
 	{ "SEND_IGNORE_MSG",		BOOL_TYPE_VAR,	DEFAULT_SEND_IGNORE_MSG, NULL, NULL, 0, 0 },
+/**************************** PATCHED by Flier ******************************/
+        { "SEND_USERHOST_ON_NICK_IN_USE",BOOL_TYPE_VAR, 0, NULL, NULL, 0, 0 },
+/****************************************************************************/
 	{ "SHELL",			STR_TYPE_VAR,	0, NULL, NULL, 0, VF_NODAEMON },
 	{ "SHELL_FLAGS",		STR_TYPE_VAR,	0, NULL, NULL, 0, VF_NODAEMON },
 	{ "SHELL_LIMIT",		INT_TYPE_VAR,	DEFAULT_SHELL_LIMIT, NULL, NULL, 0, VF_NODAEMON },
@@ -286,11 +300,14 @@ static	IrcVariable irc_variable[] =
 	{ "STATUS_HOLD_LINES",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
 	{ "STATUS_INSERT",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
 /**************************** PATCHED by Flier ******************************/
+        { "STATUS_LINES",		INT_TYPE_VAR,	DEFAULT_STATUS_LINES, NULL, SetStatusLines, 0, 0 },
         { "STATUS_LOADAVG",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
 /****************************************************************************/
 	{ "STATUS_MAIL",		STR_TYPE_VAR,	0, NULL, build_status, 0, VF_NODAEMON },
 	{ "STATUS_MODE",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
 	{ "STATUS_NOTIFY",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
+	{ "STATUS_NOTIFY_REPW_END_VAR",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
+	{ "STATUS_NOTIFY_REPW_START_VAR",	STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
 	{ "STATUS_OPER",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
 	{ "STATUS_OVERWRITE",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
 	{ "STATUS_QUERY",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
@@ -301,6 +318,8 @@ static	IrcVariable irc_variable[] =
         { "STATUS_REVERSE",		BOOL_TYPE_VAR,	1, NULL, build_status, 0, 0 },
 #endif
 /****************************************************************************/
+	{ "STATUS_SCROLLED",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
+	{ "STATUS_SCROLLED_LINES",	STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
 	{ "STATUS_SERVER",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
 	{ "STATUS_UMODE",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
 /**************************** PATCHED by Flier ******************************/
@@ -326,7 +345,9 @@ static	IrcVariable irc_variable[] =
         { "TRUNCATE_PUBLIC_CHANNEL",	BOOL_TYPE_VAR, 	DEFAULT_TRUNCATE_PUBLIC_CHANNEL, NULL, NULL, 0, 0 },
 /****************************************************************************/
 	{ "UNDERLINE_VIDEO",		BOOL_TYPE_VAR,	DEFAULT_UNDERLINE_VIDEO, NULL, NULL, 0, 0 },
-	{ "USE_OLD_MSG",		BOOL_TYPE_VAR,	DEFAULT_USE_OLD_MSG, NULL, NULL, 0, 0 },
+/**************************** PATCHED by Flier ******************************/
+        { "USERNAME",		STR_TYPE_VAR,	0, NULL, SetUsername, 0, 0 },
+/****************************************************************************/
 	{ "USER_INFORMATION", 		STR_TYPE_VAR,	0, NULL, NULL, 0, 0 },
 	{ "USER_WALLOPS",		BOOL_TYPE_VAR,	DEFAULT_USER_WALLOPS, NULL, NULL, 0, 0 },
 	{ "VERBOSE_CTCP",		BOOL_TYPE_VAR,	DEFAULT_VERBOSE_CTCP, NULL, NULL, 0, 0 },
@@ -348,6 +369,8 @@ init_variables()
         int old_disp;
 /****************************************************************************/
 
+	check_variable_order();
+
 	set_string_var(CMDCHARS_VAR, DEFAULT_CMDCHARS);
 	set_string_var(LOGFILE_VAR, DEFAULT_LOGFILE);
 	set_string_var(SHELL_VAR, DEFAULT_SHELL);
@@ -361,10 +384,10 @@ init_variables()
 	set_string_var(IRC_ENCODING_VAR, DEFAULT_IRC_ENCODING);
 	set_string_var(HIGHLIGHT_CHAR_VAR, DEFAULT_HIGHLIGHT_CHAR);
 	set_string_var(HISTORY_FILE_VAR, DEFAULT_HISTORY_FILE);
-	set_string_var(IRCHOST_VAR, empty_string);
 	set_string_var(LASTLOG_LEVEL_VAR, DEFAULT_LASTLOG_LEVEL);
 	set_string_var(NOTIFY_HANDLER_VAR, DEFAULT_NOTIFY_HANDLER);
 	set_string_var(NOTIFY_LEVEL_VAR, DEFAULT_NOTIFY_LEVEL);
+	set_string_var(RATE_LIMIT_JOIN_VAR, DEFAULT_RATE_LIMIT_JOIN);
 	set_string_var(REALNAME_VAR, realname);
 	set_string_var(STAMP_FORMAT, DEFAULT_STAMP_FORMAT);
 	set_string_var(STATUS_FORMAT_VAR, DEFAULT_STATUS_FORMAT);
@@ -391,6 +414,8 @@ init_variables()
 	set_string_var(STATUS_OPER_VAR, DEFAULT_STATUS_OPER);
 	set_string_var(STATUS_OVERWRITE_VAR, DEFAULT_STATUS_OVERWRITE);
 	set_string_var(STATUS_QUERY_VAR, DEFAULT_STATUS_QUERY);
+	set_string_var(STATUS_SCROLLED_VAR, DEFAULT_STATUS_SCROLLED);
+	set_string_var(STATUS_SCROLLED_LINES_VAR, DEFAULT_STATUS_SCROLLED_LINES);
 	set_string_var(STATUS_SERVER_VAR, DEFAULT_STATUS_SERVER);
 	set_string_var(STATUS_UMODE_VAR, DEFAULT_STATUS_UMODE);
 	set_string_var(STATUS_USER_VAR, DEFAULT_STATUS_USER);
@@ -445,6 +470,24 @@ init_variables()
 /****************************************************************************/
 }
 
+/*
+ * check_variable_order: make sure irc_variable[] is properly ordered.
+ */
+static  void
+check_variable_order(void)
+{
+	IrcVariable *curr, *prev = NULL;
+	size_t  len;
+
+	for (curr = irc_variable; curr->name; prev = curr++)
+		if (prev && (len = strlen(prev->name)) && strncmp(prev->name, curr->name, len) > 0)
+		{
+			yell("irc_variables[] order is wrong at elements \"%s\" and \"%s\"!", prev->name, curr->name);
+			abort();
+		}
+
+}
+
 /*
  * find_variable: looks up variable name in the variable table and returns
  * the index into the variable array of the match.  If there is no match, cnt
@@ -963,6 +1006,22 @@ int value;
     set_int_var(DCC_BLOCK_SIZE_VAR,value);
 }
 
+static void SetDCCHost(value)
+char *value;
+{
+    if (value && *value) {
+        struct in_addr inp;
+        struct hostent *hp;
+
+        if (inet_aton(value, &inp) == 0) {
+            if ((hp = gethostbyname(value)) != NULL)
+                bcopy(hp->h_addr, (char *) &forced_ip_addr, sizeof(forced_ip_addr));
+            else say("DNS lookup for %s failed", value);
+        }
+        else bcopy(&inp, (char *) &forced_ip_addr, sizeof(forced_ip_addr));
+    }
+}
+
 static void SetDCCPorts(value)
 char *value;
 {
@@ -1044,4 +1103,20 @@ char *tsformat;
                                    empty_string, &flag, NULL);
 #endif /* HAVE_STRFTIME */
 }
+
+static void SetStatusLines(value)
+int value;
+{
+    if (value > 3) value = 3;
+    if (value < 1) value = 1;
+    set_int_var(STATUS_LINES_VAR, value);
+    StatusLines = value - 1;
+}
+
+static void SetUsername(new_username)
+char *new_username;
+{
+    if (new_username) strmcpy(username, new_username, NAME_LEN);
+    else strmcpy(username, empty_string, NAME_LEN);
+}
 /****************************************************************************/
